<?php

namespace Drupal\oembed\Provider;

use Bangpound\oEmbed\Provider\ProviderInterface;
use Bangpound\oEmbed\Provider\StandardProvider;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\oembed\Entity\Provider;

/**
 *
 */
class ConfigEntityProvider implements ProviderInterface {
  /**
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $providerStorage;

  /**
   * @var array
   */
  protected $providers = [];

  /**
   *
   */
  public function __construct(EntityTypeManagerInterface $entityManager) {
    $this->providerStorage = $entityManager
      ->getStorage('oembed_provider');
  }

  /**
   * @param $url
   * @param array $params
   *
   * @return \Psr\Http\Message\RequestInterface
   */
  public function request($url, array $params = []) {
    $provider_name = '';
    if (empty($this->providers)) {
      $this->loadProviders();
    }

    $schemes = self::getSchemes($this->providers);
    foreach ($schemes as $name => $scheme) {
      foreach ($scheme as $value) {
        $regex = self::schemeToRegex($value);
        if (preg_match($regex, $url)) {
          $provider_name = $name;
          break 2;
        }
      }
    }

    /** @var \Drupal\oembed\Entity\Provider $providerEntity */
    $providerEntity = $this->providers[$provider_name];
    $provider = new StandardProvider(
      $providerEntity->getEndpoint(),
      $providerEntity->getScheme(),
      [],
      ['format' => 'json']
    );
    return $provider->request($url, $params);
  }

  /**
   * Returns whether this class supports the given url.
   *
   * @param string $url
   *   A url.
   * @param array $params
   *   The resource type or null if unknown.
   *
   * @return bool True if this class supports the given url, false otherwise
   */
  public function supports($url, array $params = []) {
    if (empty($this->providers)) {
      $this->loadProviders();
    }

    $schemes = array_merge(array_values(self::getSchemes($this->providers)));
    foreach ($schemes as $scheme) {
      foreach ($scheme as $value) {
        $regex = self::schemeToRegex($value);
        if (preg_match($regex, $url)) {
          return TRUE;
        }
      }
    }

    return FALSE;
  }

  /**
   *
   */
  private function loadProviders() {
    $this->providers = $this->providerStorage->loadByProperties(['status' => TRUE]);
  }

  /**
   * @param $providers
   * @return array
   */
  private static function getSchemes($providers) {
    return array_map(function (Provider $provider) {
      return $provider->getScheme();
    }, $providers);
  }

  /**
   * Changes oEmbed schemes to regular expressions.
   *
   * @param $scheme
   * @param bool $capture_subpatterns
   *
   * @return mixed|string
   */
  private static function schemeToRegex($scheme, $capture_subpatterns = FALSE) {
    $patterns = [];
    $schemes = array_filter(preg_split("/(\r\n?|\n)/", $scheme));
    foreach ($schemes as $subject) {
      $patterns[] = str_replace('\*', $capture_subpatterns ? '(.*)' : '.*', preg_quote($subject, '#'));
    }

    return '#' . implode('|', $patterns) . '#i';
  }

}
